use std::sync::Arc;

use serde::Deserialize;
use tokio::sync::OnceCell;

pub static CONFIG: OnceCell<Arc<Config>> = OnceCell::const_new();

/// Get a reference to global config
pub fn get_config() -> &'static Arc<Config> {
    crate::conf::CONFIG.get().unwrap()
}

/// Load a global config
pub fn load_config(path: &str) {
    // TODO : Other load locations
    if let Ok(file_content) = std::fs::read_to_string(path) {
        let conf: Config =
            toml::from_str(&file_content).expect("Could not deserialize config file");
        crate::conf::CONFIG.set(std::sync::Arc::new(conf)).unwrap();
    }
}

/// Load a default global config
pub fn load_default_config() {
    if crate::conf::CONFIG.get().is_none() {
        crate::conf::CONFIG
            .set(std::sync::Arc::new(Config::default()))
            .unwrap();
    }
}

#[allow(non_snake_case)]
#[derive(Debug, Clone, Deserialize)]
pub struct Config {
    pub ROUTE_INTERNAL: bool,
    pub DOWNLOAD_ON_DEMAND: bool,
    pub ai: Option<AIConfig>,
    pub websites: Option<WebsiteConfig>,
}

impl Config {
    pub fn get_domain_config(&self, domain: &str) -> Option<&DomainConfig> {
        if let Some(websites) = &self.websites {
            if let Some(domains) = &websites.domains {
                let domain = domains.iter().find(|x| x.domain == domain);
                return domain;
            }
        }

        None
    }
}

#[allow(non_snake_case)]
#[derive(Debug, Clone, Deserialize)]
pub struct AIConfig {
    pub OLLAMA_URL: String,
}

#[allow(non_snake_case)]
#[derive(Debug, Clone, Deserialize)]
pub struct WebsiteConfig {
    pub BLACKLIST_DOMAINS: Option<Vec<String>>,
    pub domains: Option<Vec<DomainConfig>>,
}

#[derive(Debug, Clone, Deserialize)]
pub struct DomainConfig {
    pub domain: String,
    pub blacklist_paths: Option<Vec<String>>,
    pub no_audio: bool,
    pub no_video: bool,
    pub no_image: bool,
    pub no_css: bool,
    pub no_javascript: bool,
    pub no_fonts: bool,
    pub no_frames: bool,
    pub user_agent: Option<String>,
}

impl Default for DomainConfig {
    fn default() -> Self {
        Self {
            domain: String::new(),
            blacklist_paths: None,
            no_audio: false,
            no_video: false,
            no_image: false,
            no_css: false,
            no_javascript: false,
            no_fonts: false,
            no_frames: false,
            user_agent: None,
        }
    }
}

impl Default for Config {
    fn default() -> Self {
        Self {
            ROUTE_INTERNAL: false,
            DOWNLOAD_ON_DEMAND: false,
            ai: None,
            websites: None,
        }
    }
}
